function challenge(record, debug)
%challenge assesses one of the CinC 2014 Challenge records
%
%                                  DISCLAIMER                                  
% This software and documentation (the "Software") were developed at the Food  
% and Drug Administration (FDA) by employees of the Federal Government in the  
% course of their official duties. Pursuant to Title 17, Section 105 of the    
% United States Code, this work is not subject to copyright protection and is  
% in the public domain. Permission is hereby granted, free of charge, to any   
% person obtaining a copy of the Software, to deal in the Software without     
% restriction, including without limitation the rights to use, copy, modify,   
% merge, publish, distribute, sublicense, or sell copies of the Software or    
% derivatives, and to permit persons to whom the Software is furnished to do   
% so. FDA assumes no responsibility whatsoever for use by other parties of     
% the Software, its source code, documentation or compiled executables, and    
% makes no guarantees, expressed or implied, about its quality, reliability,   
%  or any other characteristic. Further, use of this code in no way implies    
% endorsement by the FDA or confers any advantage in regulatory decisions.     
% Although this software can be redistributed and/or modified freely, we       
% ask that any derivative works bear some notice that they are derived from    
% it, and any modified versions bear some notice that they have been modified. 
%                                                                              
% Written by: Jose Vicente
	pkg load signal;

	rec = loadrec(record);
	origfs = rec{1}.fs;
	[anns, out] = combined(rec, debug);

	if(~isempty(anns))
		saveann(record, origfs, 'qrs', anns);
	end

	if(debug)
		if(~isempty(out{1}.anns))
			saveann(record, origfs, 'ecg', out{1}.anns);
		end

		bp = 0;
		eeg = 0;
		emg = 0;
		eog = 0;
		sv = 0;
		ppg = 0;

		for j=2:length(out)
			if (strcmp(out{j}.type,'BP') && ~isempty(out{j}.anns) && bp == 0)
				saveann(record, origfs, 'bp', out{j}.anns);
				bp = j;
			elseif (strcmp(out{j}.type,'EEG') && ~isempty(out{j}.anns) && eeg == 0)
				saveann(record, origfs, 'eeg', out{j}.anns);
				eeg = j;
			elseif (strcmp(out{j}.type,'SV') && ~isempty(out{j}.anns) && sv == 0)
				saveann(record, origfs, 'sv', out{j}.anns);
				sv = j;
			elseif (strcmp(out{j}.type,'EMG') && ~isempty(out{j}.anns) && emg == 0)
				saveann(record, origfs, 'emg', out{j}.anns);
				emg = j;
			elseif (strcmp(out{j}.type,'EOG') && ~isempty(out{j}.anns) && eog == 0)
				saveann(record, origfs, 'eog', out{j}.anns);
				eog = j;
			elseif (strcmp(out{j}.type,'PPG') && ~isempty(out{j}.anns) && eog == 0)
				saveann(record, origfs, 'ppg', out{j}.anns);
				eog = j;
			end
		end

		onam = strcat(record,'.mat');
		save(onam,'anns','out','origfs');
	end
end
