%% @file 
% ecg_sqi.m
%
% @author Lars Johannesen
%
% @section DISCLAIMER
% This software and documentation (the "Software") were developed at the Food
% and Drug Administration (FDA) by employees of the Federal Government in the
% course of their official duties. Pursuant to Title 17, Section 105 of the
% United States Code, this work is not subject to copyright protection and is
% in the public domain. Permission is hereby granted, free of charge, to any
% person obtaining a copy of the Software, to deal in the Software without
% restriction, including without limitation the rights to use, copy, modify,
% merge, publish, distribute, sublicense, or sell copies of the Software or
% derivatives, and to permit persons to whom the Software is furnished to do
% so. FDA assumes no responsibility whatsoever for use by other parties of
% the Software, its source code, documentation or compiled executables, and
% makes no guarantees, expressed or implied, about its quality, reliability,
%  or any other characteristic. Further, use of this code in no way implies
% endorsement by the FDA or confers any advantage in regulatory decisions.
% Although this software can be redistributed and/or modified freely, we
% ask that any derivative works bear some notice that they are derived from
% it, and any modified versions bear some notice that they have been modified.
%
% @section Examples:
%  [sqi] = ecg_sqi(ann, dbg,maxdiff,minsnr)
%
% @section See also:
%

%% 
% @brief Determines which ecg annotations are reliable
% @param ann Annotations
% @param dbg Debug output form the qrsdetector
% @param maxdiff If two annotations are within maxdiff they are likely noisy (same units as annotation)
% @param minsnr Minimum qrsdetector signal/noise ratio
% @return [sqi] = SQI vector
%
function [sqi] = ecg_sqi(anns, dbg, maxdiff, minsnr)
%ecg_sqi Determines which ecg annotations are reliable
%
% ecg_sqi(ann, dbg, maxdiff, minsnr)
%
% ann - annotation
% dbg - debug output from qrsdetector
% maxdiff - If two annotations are within maxdiff they are likely noisy (same units as annotation)
% minsnr Minimum qrsdetector signal/noise ratio
%
%                                  DISCLAIMER
% This software and documentation (the "Software") were developed at the Food
% and Drug Administration (FDA) by employees of the Federal Government in the
% course of their official duties. Pursuant to Title 17, Section 105 of the
% United States Code, this work is not subject to copyright protection and is
% in the public domain. Permission is hereby granted, free of charge, to any
% person obtaining a copy of the Software, to deal in the Software without
% restriction, including without limitation the rights to use, copy, modify,
% merge, publish, distribute, sublicense, or sell copies of the Software or
% derivatives, and to permit persons to whom the Software is furnished to do
% so. FDA assumes no responsibility whatsoever for use by other parties of
% the Software, its source code, documentation or compiled executables, and
% makes no guarantees, expressed or implied, about its quality, reliability,
%  or any other characteristic. Further, use of this code in no way implies
% endorsement by the FDA or confers any advantage in regulatory decisions.
% Although this software can be redistributed and/or modified freely, we
% ask that any derivative works bear some notice that they are derived from
% it, and any modified versions bear some notice that they have been modified.
%
% Written by: Lars Johannesen
%
% Examples:
%  [sqi] = ecg_sqi(ann, dbg,maxdiff,minsnr)
%
% See also:
	anns = anns(:);
	dfs = diff(anns);

	% Create vector of same length as anns when the 
	dfs = [0 (dfs<maxdiff)'];
	dfs_shift = [dfs(2:end) 0];

	sqi1 = ~(dfs | dfs_shift);
	tsqi2 = dbg{1}(:,2)./dbg{1}(:,3);
	tsqi2(dbg{1}(:,3)==0) = 100;
	sqi2 = interp1(dbg{1}(:,1), tsqi2, anns);

	sqi3 = sqi1' & sqi2>=minsnr;
	sqi = zeros(size(sqi3));
	sqi(sqi3) = 1;
end
