/********************************************************************************
 *                                  DISCLAIMER                                  *
 * This software and documentation (the "Software") were developed at the Food  *
 * and Drug Administration (FDA) by employees of the Federal Government in the  *
 * course of their official duties. Pursuant to Title 17, Section 105 of the    *
 * United States Code, this work is not subject to copyright protection and is  *
 * in the public domain. Permission is hereby granted, free of charge, to any   *
 * person obtaining a copy of the Software, to deal in the Software without     *
 * restriction, including without limitation the rights to use, copy, modify,   *
 * merge, publish, distribute, sublicense, or sell copies of the Software or    *
 * derivatives, and to permit persons to whom the Software is furnished to do   *
 * so. FDA assumes no responsibility whatsoever for use by other parties of     *
 * the Software, its source code, documentation or compiled executables, and    *
 * makes no guarantees, expressed or implied, about its quality, reliability,   *
 *  or any other characteristic. Further, use of this code in no way implies    *
 * endorsement by the FDA or confers any advantage in regulatory decisions.     *
 * Although this software can be redistributed and/or modified freely, we       *
 * ask that any derivative works bear some notice that they are derived from    *
 * it, and any modified versions bear some notice that they have been modified. * 
 *                                                                              *
 * Written by:                                                                  *
 *  Lars Johannesen                                                             *
 ********************************************************************************/
#include "mex.h"

#include <wfdb/wfdb.h>

#include <vector>
#include <map>
#include <string>

#include <iostream>

#include <stdexcept>

#include <cmath>

mxArray *load_ann(const mxArray *signam, const mxArray *annnam, const double fs);

template<typename TYP>
TYP get_matlab_scalar(const mxArray *a) {
        TYP *t = (TYP*)mxGetData(a);
        return *t;
}

void mexFunction (int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {
	if(nrhs != 3) {
		mexErrMsgTxt("Need one arguments: record fs ann");
	}

	try {
		if(!mxIsChar(prhs[0]))
                        mexErrMsgTxt("First argument must be a char");

		if(!mxIsDouble(prhs[1]))
                        mexErrMsgTxt("Second argument must be a double");

		if(!mxIsChar(prhs[2]))
                        mexErrMsgTxt("Third argument must be a char");

		double fs = get_matlab_scalar<double>(prhs[1]);
		
		mxArray *sig = load_ann(prhs[0], prhs[2], fs);
		plhs[0] = sig;
	} catch (...) {
		mexErrMsgTxt("Failed loading");
	}
}

mxArray *load_ann(const mxArray *signam, const mxArray *annnam, const double fs) {
	char *rec = mxArrayToString(signam);
	char *ann = mxArrayToString(annnam);

	WFDB_Anninfo a;
	WFDB_Annotation annot;

	a.name = ann;
	a.stat = WFDB_READ;
	
	if(annopen(rec, &a, 1) < 0) {
		throw std::runtime_error("Arrr");
	}

	std::vector<WFDB_Annotation> anns;

	while(getann(0, &annot)==0) { 
		anns.push_back(annot);
	}

	mwSize dims[1];
	dims[0] = anns.size();

	mxArray *annsp = mxCreateNumericArray(1, dims, mxDOUBLE_CLASS, mxREAL);
	double *ptr = (double*) mxGetPr(annsp);
	for(std::size_t i = 0; i < anns.size(); ++i) {
		ptr[i] = static_cast<double>(round(anns[i].time*1000./fs));
	}

	return annsp;
}
